#include "DFS_online_cycle_detector.hpp"

using namespace std;

DFS_online_cycle_detector::DFS_online_cycle_detector(graph2_t &g,
						     vector<variable_info> &v)
  : _graph(g), _varmap(v), _data(size(g)*2,false),
    _vindex(size(g)), _index(0), _n2i(size(g)), _invalid(true) { 
  std::vector<unsigned int> q;
  finish(q);
  assert(q.empty());
}

#define VISITED(x) _data[(x<<1)]
#define IN_COMPONENT(x) _data[(x<<1)+1]

void DFS_online_cycle_detector::finish(std::vector<unsigned int> &q) {
  if(_invalid) {
    _i2c.clear();
    _index=0;
    for(unsigned int i=0;i!=size(_graph);++i) {
      if(_varmap[i].node == i && !VISITED(i)) {
	cycle_detect(i,q);
      }
    }
    fill(_data.begin(),_data.end(),false);
    _invalid=false;
  }
}

void DFS_online_cycle_detector::new_edge(unsigned int tail, unsigned int head,
				     std::vector<unsigned int> &q) {
  // remember that n2i is in reverse order!
  if(_n2i[head] > _n2i[tail]) { _invalid=true; }
}

void DFS_online_cycle_detector::cycle_detect(unsigned int n,
					 std::vector<unsigned int> &q) {
  assert(_varmap[n].node == n);
  VISITED(n)=true;
  IN_COMPONENT(n)=false;
  _vindex[n]=_index++;
  // now process all successors
  graph1_t::const_siterator i,iend;
  for(boost::tie(i,iend) = successors(n,_graph);i!=iend;++i) {
    unsigned int w(*i);        
    if(!VISITED(w)) { cycle_detect(w,q); }
    if(!IN_COMPONENT(w)) {
      unsigned int t(_varmap[w].node);
      if(_vindex[t] < _vindex[_varmap[n].node]) {
	_varmap[n].node = t;
      }
    }
  }
  
  // at this point we must check to see
  // if any components have been identified
  if(_varmap[n].node == n) {
    unsigned int t(_vindex[n]);
    IN_COMPONENT(n) = true;

    _n2i[n] = _i2c.size();
    _i2c.push_back(n);

    while(!_stack.empty() && t < _vindex[_stack.back()]) {
      unsigned int w(_stack.back());
      _varmap[w].node = n;
      IN_COMPONENT(w) = true;
      q.push_back(w);
      _stack.pop_back(); 
    }
  } else {
    _stack.push_back(n);
  }
}









