#include <iostream>
#include "MNR_online_cycle_detector.hpp"
#include "topological_sort.hpp"

#if __GNUC__ >= 3
#include <ext/functional>
#define SELECT1ST __gnu_cxx::select1st
#define PROJECT1ST __gnu_cxx::project1st
#else 
// everything else for the moment.
#include <functional>
#define SELECT1ST std::select1st
#define PROJECT1ST std::project1st
#endif

using namespace std;

MNR_online_cycle_detector::MNR_online_cycle_detector(graph2_t &g,
						     vector<variable_info> &v)
  : _graph(g), _varmap(v), _i2c(size(g)), _c2i(size(g)), _data(2*size(g),false) {
  // pre: graph is a DAG
  std::vector<unsigned int> order(size(g));
  // topologically sort graph.
  
  topological_sort(_graph,order);
  unsigned int index(0);
  // remember topological sort is produced
  // in reverse order.
  std::vector<unsigned int>::reverse_iterator iend(order.rend());
  for(std::vector<unsigned int>::reverse_iterator i(order.rbegin());
      i!=iend;++i,++index) {
    _c2i[*i]=index;
    _i2c[index]=*i;
  }  
}

#define VISITED(x) _data[(x<<1)]
#define IN_COMPONENT(x) _data[(x<<1)+1]
#define C2I(x) _c2i[x]
#define I2C(x) _i2c[x]

void MNR_online_cycle_detector::new_edge(unsigned int tail, unsigned int head,
					 std::vector<unsigned int> &q) {
  unsigned int lb(C2I(head));
  unsigned int ub(C2I(tail));	

  if(lb < ub) {
    // in this case we are forced
    // to update the topological
    // order
    VISITED(tail)=true;
    IN_COMPONENT(tail) = true;
    // should be able to replace these with
    // generic call to depth_first_visit
    dfs_f(head,ub);
    reorder(lb,ub,q);
  }
}

void MNR_online_cycle_detector::dfs_f(unsigned int n, unsigned int ub) {	   
  assert(n < size(_graph));
  VISITED(n)=true;    

  graph1_t::const_siterator i,iend;
  for(boost::tie(i,iend) = successors(n,_graph);i!=iend;++i) {
    assert(C2I(*i) >= C2I(n) || C2I(n) == ub);
    if(C2I(*i) <= ub) {
      if(!VISITED(*i)) { dfs_f(*i,ub); }
      IN_COMPONENT(n) = IN_COMPONENT(n) | IN_COMPONENT(*i);
    } 
  }
}

void MNR_online_cycle_detector::reorder(unsigned int lb,
					unsigned int ub,
					std::vector<unsigned int> &q) {  
  unsigned int shift(0);
  unsigned int tailc(I2C(ub));
  vector<unsigned int> tmp; // temporary storage 
  unsigned int i;
  bool prep(false);
  IN_COMPONENT(tailc)=false;
  VISITED(tailc)=false;
  
  for(i=lb;i<=ub;++i) {
    unsigned int c(I2C(i));
    assert(C2I(c) == i);

    bool v(VISITED(c));
    bool ic(IN_COMPONENT(c));
    if(v && !ic) {
      // we must push on the blanks as well ...
      tmp.push_back(c);
      ++shift;
      VISITED(c) = false;
    } else {
      if(ic) {
	VISITED(c) = false;
	IN_COMPONENT(c) = false;
	_varmap[c].node = tailc;
	q.push_back(c);
      }
      C2I(c) = i-shift;
      I2C(i-shift) = c;
    }
  }
  
  i-=shift;
  for(unsigned int j=0;j<tmp.size();++j) {
    C2I(tmp[j]) = i+j;
    I2C(i+j) = tmp[j];      
  }
}









