#include <sys/time.h>
#include <set>
// unsure why this is needed
#define BOOST_NO_STDC_NAMESPACE
#include <boost/random.hpp>
#include <boost/lexical_cast.hpp>

#include "bitset.hpp"

using namespace std;

template<class T>
string set2str(T const &s) {
  string r;
  for(typename T::const_iterator i(s.begin());
      i!=s.end();++i) {
    if(i != s.begin()) r += ", ";
    r += boost::lexical_cast<string>(*i);
  }
  return r;
}

bool compare(set<unsigned int> &sset, bitset &bset) {
  if(sset.size() != bset.size()) {
    return false;
  }
  bitset::iterator i(bset.begin());
  for(set<unsigned int>::iterator j(sset.begin());
      j!=sset.end();++j,++i) {
    if(i == bset.end() || *i != *j) {
      return false;
    }    
  }
  return true;
}

int main(int argc,char *argv[]) {
  boost::mt19937 urng;
  boost::random_number_generator<boost::mt19937,unsigned int> rng(urng);
  // seed the random number generator
  struct timeval tmp;
  gettimeofday(&tmp,NULL);
  urng.seed((tmp.tv_sec * 1000000) + tmp.tv_usec); 

  set<unsigned int> s1set,s2set;
  bitset b1set,b2set;;
  bool r1,r2;

  for(unsigned int i=0;i!=1000;++i) {
    b1set.clear();b2set.clear();
    s1set.clear();s2set.clear();
    for(unsigned int j=0;j!=10000;++j) {
      unsigned int v(rng(100000));
      std::pair<bitset::iterator,bool> t1;
      std::pair<set<unsigned int>::iterator,bool> t2;
      t1 = b1set.insert(v);      
      t2 = s1set.insert(v);
      
      if(!compare(s1set,b1set) || t1.second != t2.second || *t1.first != v) {
	cout << "!!! Failure: inserted " << v << endl;
	cout << "\tstd::set [" <<  s1set.size() << "] = " << set2str(s1set) << endl;
	cout << "\tbitset [" <<  b1set.size() << "] = " << set2str(b1set) << endl;
      }

      if(rng(100) > 50) {
	b1set.erase(t1.first);
	s1set.erase(t2.first);	
	if(!compare(s1set,b1set)) {
	  cout << "!!! Failure: erasing from iterator " << v << endl;
	  cout << "\tstd::set [" <<  s1set.size() << "] = " << set2str(s1set) << endl;
	  cout << "\tbitset [" <<  b1set.size() << "] = " << set2str(b1set) << endl;
	}
      }
    }

    for(unsigned int j=0;j!=10000;++j) {
      unsigned int v(rng(100000));
      r1 = b2set.insert(v).second;
      r2 = s2set.insert(v).second;
      
      if(!compare(s2set,b2set) || r1 != r2) {
	cout << "!!! Failure: inserted " << v << endl;
	cout << "\tstd::set [" << s2set.size() << "] = " << set2str(s2set) << endl;
	cout << "\tbitset [" << b2set.size() << "] = " << set2str(b2set) << endl;
      }
    }
    r1 = set_union(b1set,b2set);
    r2 = set_union(s1set,s2set);
    if(!compare(s1set,b1set) || r1!=r2) {
      cout << "!!! Failure: set_union " << endl;
      cout << "\tstd::set [" <<  s1set.size() << "] = " << set2str(s1set) << endl;
      cout << "\tbitset [" << b1set.size() << "] = " << set2str(b1set) << endl;
    }

    // now test erasing ...
    for(unsigned int j=0;j!=10000;++j) {
      unsigned int v(rng(100000));
      r1 = b1set.erase(v) == 1;
      r2 = s1set.erase(v) == 1;
      
      if(!compare(s1set,b1set) || r1 != r2) {
	cout << "!!! Failure: erasing " << v << endl;
	cout << "\tstd::set [" <<  s1set.size() << "] = " << set2str(s1set) << endl;
	cout << "\tbitset [" <<  b1set.size() << "] = " << set2str(b1set) << endl;
      }
    }

    // finally test finding ...
    for(unsigned int j=0;j!=10000;++j) {
      unsigned int v(rng(100000));
      r1 = b1set.find(v) != b1set.end();
      r2 = s1set.find(v) != s1set.end();
      
      if(r1 != r2) {
	cout << "!!! Failure: finding " << v << endl;
	cout << "\tstd::set [" <<  s1set.size() << "] = " << set2str(s1set) << endl;
	cout << "\tbitset [" <<  b1set.size() << "] = " << set2str(b1set) << endl;
      }
    }
  }
  
  exit(0);
}



