#ifndef DEPTH_FIRST_SEARCH_HPP
#define DEPTH_FIRST_SEARCH_HPP

#include <boost/tuple/tuple.hpp>
#include <vector>

template<class Graph, class VisitedMap, class Visitor>
class depth_first_searcher {
 protected:
  Graph const &_g;
  VisitedMap _visited;
  Visitor _visitor;
 public:
  depth_first_searcher(Graph const &g, VisitedMap visited, Visitor visitor) 
    : _g(g), _visited(visited), _visitor(visitor) {
  }
  
  void visit(unsigned int n) {
    _visited[n]=true;
    // fire event
    _visitor.start_vertex(n);
    typename Graph::const_siterator i,iend;
    for(boost::tie(i,iend) = successors(n,_g);
	i!=iend;++i) {
      if(!_visited[*i]) {
	_visitor.traversing_edge(n,*i);
	visit(*i);
	_visitor.finished_edge(n,*i);
      } else {
	_visitor.skipping_edge(n,*i);
      }
    }
    _visitor.finished_vertex(n);
  }
};

template<class Graph, class VisitedMap, class Visitor>
void depth_first_search(unsigned int n, Graph const &g, VisitedMap &visited, Visitor visitor) {
  depth_first_searcher<Graph,VisitedMap&,Visitor> searcher(g,visited,visitor);
  searcher.visit(n);
}

template<class Graph, class Visitor>
void depth_first_search(unsigned int n, Graph const &g, Visitor visitor) {
  depth_first_searcher<Graph,std::vector<bool>,Visitor> searcher(g,visitor);
  searcher.visit(n);
}

#endif
