#ifndef BMNR_ONLINE_CYCLE_DETECTOR_HPP
#define BMNR_ONLINE_CYCLE_DETECTOR_HPP

#include <utility>
#include <vector>
#include "pcs.hpp" // for zero_edge_select
#include "bidirectional_adjacency_list.hpp"
#include "filter_graph.hpp"
#include "cycle_detector.hpp"

class BMNR_online_cycle_detector : public cycle_detector {
  typedef bidirectional_adjacency_list<vector_set<unsigned int> > graph2_t;
  typedef filter_graph<graph2_t,zero_edge_select> graph1_t;
public:  
  graph1_t _graph; // the filtered graph
  std::vector<variable_info> &_varmap;
  std::vector<unsigned int> _vindex;  // visitation index
  std::vector<unsigned int> _stack;   // used to associate nodes with components
  std::vector<unsigned int> _c2i;     // component to index map
  std::vector<unsigned int> _i2c;     // index to component map
  std::vector<unsigned int> _data;    // holds visited and in_component
  std::vector<std::pair<unsigned int, unsigned int> > _batch;         // holds the batch of edges to be processed
  unsigned int _index;
public:
  BMNR_online_cycle_detector(graph2_t &, std::vector<variable_info> &);

  void new_edge(unsigned int, unsigned int, std::vector<unsigned int> &);
  void finish(std::vector<unsigned int> &);
private:
  void dfs_f(unsigned int, unsigned int,
	     std::vector<std::pair<unsigned int, unsigned int> > &,
	     std::vector<unsigned int> &);
  void shift(unsigned int,
	     std::vector<std::pair<unsigned int, unsigned int> > &);
};

#endif








