#ifndef OLD_WORKLIST_SOLVER_HPP
#define OLD_WORKLIST_SOLVER_HPP

#include <set>
#include <string>
#include <boost/tuple/tuple.hpp>
#include "diedge.hpp"
#include "pcs.hpp" 
#include "set_union.hpp"
#include "store.hpp"
#include "worklist_strategy.hpp"
#include "solver_common.hpp"
#include "cycle_detector.hpp"

template<class G, class S, class B>
class old_worklist_solver_common : public B {
private:
  worklist_strategy &_worklist;
  cycle_detector &_cycle_detector; 
public:
  old_worklist_solver_common(G &g, std::vector<variable_info> &v, 
			     store<S> &s, worklist_strategy &strategy,
			     cycle_detector &cd, 
			     checker_stats &stats,
			     bool ftc = false) 
    : B(g,v,s,stats,ftc), _worklist(strategy), 
      _cycle_detector(cd) {
    //    assert(_sol.size() == size(_graph));
  }

  void solve(void) {
    // push all nodes onto the worklist
    for(unsigned int i=0;i!=size(_graph);++i) {
      if(_varmap[i].node == i) {
	_worklist.push(i);
      }
    }

    // now iterate until the worklist is empty
    while(!_worklist.empty()) {
      unsigned int c(_worklist.top()); // next
      assert(_varmap[c].node == c);
      _worklist.pop();
      enter_vertex(c); 
      _stats.visit_count++;
      // process all complex constraints involving c
      vector_set<constraint> comps(_varmap[c].complicates);
      vector_set<constraint>::const_iterator iend(comps.end());
      for(vector_set<constraint>::const_iterator i(comps.begin());
	  i!=iend;++i) {	
	do_complex_constraint(*i,_delta);
      }           
      // check for any new cycles
      do_detect_cycles();      
      // now propagate solution from c to all its successors
      typename G::const_siterator is, isend;
      for(boost::tie(is,isend) = successors(c,_graph);is!=isend;++is) {
	// if solution has changed then put
	// node back onto the worklist ...
	vector_set<unsigned int> &labels(*(is.base()->second()));
	unsigned int dest(_varmap[*is].node);
	if(do_propagate(dest,labels)) { 
	  assert(_varmap[dest].node == dest);
	  _worklist.push(dest); 
	}
      }
    }     
  }       
  
private:
  void do_detect_cycles(void) {
    // do nout for time being
    _cycle_detector.finish(_unifyqueue);
    process_unifyqueue();
  }

  bool unifynodes(unsigned int n, unsigned int np) {
    bool r = internal_unify(n,np);
    _worklist.merge(n,np);
    if(r) { _worklist.push(n); }
    return r;
  }

  void new_edge(unsigned int tail, unsigned int head, unsigned int off) {
    if(off == 0) {
      _cycle_detector.new_edge(tail,head,_unifyqueue);
      process_unifyqueue();
      head = _varmap[head].node;
      tail = _varmap[tail].node;
      if(head == tail) { return; }
    }

    if(do_new_edge(tail,head,off)) {
      _worklist.push(head);
    }
  }

  void process_trivial(unsigned int node, unsigned int val) {
    if(do_process_trivial(node,val,false)) {
      _worklist.push(node);
    }
  }
};

template<class G, class S, class B>
void old_worklist_solve(G &g, 
		    std::vector<variable_info> &v,
		    store<S> &s,
		    worklist_strategy &w,
		    cycle_detector &cd, 
		    checker_stats &stats,
		    bool f) {
  old_worklist_solver_common<G,S,B> sc(g,v,s,w,cd,stats,f);
  sc.solve();
}

#endif







