#include <set>
#include <stdexcept>
#include "vector_set.hpp"
#include "bidirectional_adjacency_list.hpp"
#include "pcs.hpp"
#include "pearce_solver.hpp"
#include "bitset.hpp"
#include "lrf_strategy.hpp"
#include "lifo_strategy.hpp"
#include "fifo_strategy.hpp"
#include "factored_store.hpp"
#include "dummy_cycle_detector.hpp"
#include "PKH_online_cycle_detector.hpp"
#include "simple_cycle_detector.hpp"
#include "simple_store.hpp"
#include "factored_store.hpp"

using namespace std;

typedef bidirectional_adjacency_list<set<unsigned int> >  graph_t;

template<class T>
store<vector_set<unsigned int> > *pearce_solvers_template(unsigned int index,
							   graph_t &g, 				     
							   vector<variable_info> &varmap,
							   checker_stats &stats,
							   bool ftc) {  

  store<T> *sol = NULL;
    
  switch(index & STORE_MASK) {
  case SIMPLE_STORE:
    sol = new simple_store<T>(size(g));
    break;    
  case FACTORED_STORE:
    sol = new factored_store<T>(size(g));
    break;    
  default:
    throw runtime_error("invalid store selected");
  }
  
  for(unsigned int i=0;i!=varmap.size();++i) {
    T tmp;
    tmp.insert(varmap[i].solution.begin(),varmap[i].solution.end());
    sol->update(i,tmp);
  }

  pearce_solve<graph_t,T>(g,varmap,*sol,stats,ftc);

  // now, create the final solution and return it ...
  store<vector_set<unsigned int> > *r = new factored_store<vector_set<unsigned int> >(size(g));

  for(unsigned int i=0;i!=varmap.size();++i) {
    if(sol->is_stored(i)) {
      vector_set<unsigned int> tmp;
      tmp.insert((*sol)[i].begin(),(*sol)[i].end());
      r->update(i,tmp);
    }
  }

  delete sol;

  return r;
}

// basically this does what the new export keyword should do.
#define INST(x,y) void pearce_solvers_ ## x(unsigned int i, \
				     graph_t &g, \
				     vector<variable_info> &v,\
                                     checker_stats &s, \
				     bool f) { pearce_solvers_template<y>(i,g,v,s,f); }

// add more instantations here ...
INST(stdset,set<unsigned int>);
INST(vecset,vector_set<unsigned int>);
INST(bitset,bitset);






