#include "find_strong_components.hpp"
#include "simple_cycle_detector.hpp"

using namespace std;

simple_cycle_detector::simple_cycle_detector(graph_t &g,
						     vector<variable_info> &v)
  : _graph(g), _varmap(v), _data(size(g)*2,false),
    _vindex(size(g)), _index(0) { 
}

#define VISITED(x) _data[(x<<1)]
#define IN_COMPONENT(x) _data[(x<<1)+1]

void simple_cycle_detector::finish(std::vector<unsigned int> &q) {
  for(std::vector<unsigned int>::iterator i(_vqueue.begin());
      i!=_vqueue.end();++i) {
    if(!VISITED(*i)) {
      cycle_detect(*i,q);
    }
  }
  _vqueue.clear();
  fill(_data.begin(),_data.end(),false);
  _index=0;
}

void simple_cycle_detector::new_edge(unsigned int tail, unsigned int head,
				     std::vector<unsigned int> &q) {
  _vqueue.push_back(tail);
}

void simple_cycle_detector::cycle_detect(unsigned int n,
					 std::vector<unsigned int> &q) {
  assert(_varmap[n].node == n);
  VISITED(n)=true;
  IN_COMPONENT(n)=false;
  _vindex[n]=_index++;
  // now process all successors
  graph_t::const_siterator i,iend;
  for(boost::tie(i,iend) = successors(n,_graph);i!=iend;++i) {
    unsigned int w(*i);
    vector_set<unsigned int> &labels(*(i.base()->second()));
    if(!labels.empty() && *labels.begin() == 0) {
      // only traverse edges with zero label
      if(!VISITED(w)) { cycle_detect(w,q); }
      if(!IN_COMPONENT(w)) {
	unsigned int t(_varmap[w].node);
	if(_vindex[t] < _vindex[_varmap[n].node]) {
	  _varmap[n].node = t;
	}
      }
    }
  }
  
  // at this point we must check to see
  // if any components have been identified
  if(_varmap[n].node == n) {
    unsigned int t(_vindex[n]);
    IN_COMPONENT(n) = true;
    while(!_stack.empty() && t < _vindex[_stack.back()]) {
      unsigned int w(_stack.back());
      _varmap[w].node = n;
      IN_COMPONENT(w) = true;
      q.push_back(w);
      _stack.pop_back(); 
    }
  } else {
    _stack.push_back(n);
  }
}




