#ifndef SIMPLE_WORKLIST_BASE_HPP
#define SIMPLE_WORKLIST_BASE_HPP

#include "solver_common.hpp"

template<class G, class S>
class simple_worklist_base : public solver_common<G,S> {
protected:
  S _delta;
  unsigned int _current;
public:
  simple_worklist_base(G &g, std::vector<variable_info> &v, store<S> &s, 
		       checker_stats &stats, bool f) 
    : solver_common<G,S>(g,v,s,stats,f) {
  }

  void enter_vertex(unsigned int n) {
    _delta = _sol[n];
  }
  void enter_vertex2(unsigned int n) {
    _delta = _sol[n];
  }

  void finish_vertex(unsigned int n) {}
  
  bool do_propagate(unsigned int dest, vector_set<unsigned int> &labels) {
    bool flag(false);
    S dest_set(_sol[dest]);
    
    for(vector_set<unsigned int>::const_iterator j(labels.begin());
	j!=labels.end();++j) {
      // propagate along edge c->*i, with label *j.      
      flag |= propagate(dest_set,_delta,*j);
    }
    
    if(flag) { _sol.update(dest,dest_set); }
    return flag;
  }

  bool internal_unify(unsigned int n, unsigned int np) {
    if(_sol.is_stored(np)) {
      S ns(_sol[n]);
      S const &nps(_sol[np]);

      _stats.union_count++;
      _stats.union_size += std::min(ns.size(),nps.size());

      if(set_union(ns,nps)) {
	_sol.update(n,ns); 
	_sol.clear(np);
	return true;
      }

      _sol.clear(np);
    }
    return false;
  }

  bool do_new_edge(unsigned int tail, unsigned int head, unsigned int off) {
    S hs(_sol[head]);
    
    _stats.edge_ucount++;
    if(!propagate(hs,_sol[tail],off)) { return false; }
    _sol.update(head,hs);
    return true;
  }

  bool do_process_trivial(unsigned int node, unsigned int val, bool mode) {
#if 1
    S const &tmp(_sol[node]);
    
    if(tmp.find(val) == tmp.end()) { 
      S ns(tmp);
      ns.insert(val);
      _sol.update(node,ns); 
      return true;
    } else {
      return false;
    }
#else
    S tmp(_sol[node]);
    
    if(tmp.insert(val).second) { 
      _sol.update(node,tmp); 
      return true;
    } else {
      return false;
    }
#endif
  }
};

#endif

