#ifndef LRF_STRATEGY_HPP
#define LRF_STRATEGY_HPP

// This graph queue exists because of the work
// by Atsushi Kanamori and Daniel Weise which 
// suggest that it is a "robust" choice when
// no static information is known about the graph.

#include <set>
#include <vector>
#include <sys/time.h>
#include <unistd.h>
#include "assert.h"
#include "bidirectional_adjacency_list.hpp"

// I have found that USE_STL_SET=1 and 
// USE_MARKED=0 is probably the best combination
// setting USE_MARKED=1 does provide a very minor
// improvement at the cost of more memory usage.

#define LRF_USE_STL_SET 1
#define LRF_USE_MARKED 0

#if LRF_USE_STL_SET == 1
class lrf_comp {
private:
  std::vector<unsigned int> &_timestamps;
public:
  lrf_comp(std::vector<unsigned int> &timestamps) 
    : _timestamps(timestamps) {    
  }

  bool operator()(unsigned int a, unsigned int b) const {
    return _timestamps[a] > _timestamps[b];
  }
};
#endif

class lrf_strategy {
private:
  std::vector<unsigned int> _timestamps;

#if LRF_USE_STL_SET == 1
  std::set<unsigned int, lrf_comp> _marks;
#else
  std::vector<unsigned int> _marks;
#endif
#if LRF_USE_MARKED == 1
  std::vector<bool> _marked;
#endif
  unsigned int _time; // the time counter
public:
  lrf_strategy(bidirectional_adjacency_list<vector_set<unsigned int> > const &);
  void push(unsigned int);
  void pop(void);
  bool is_marked(unsigned int) const;
  unsigned int top(void) const;
  bool empty(void) const;
  void merge(unsigned int, unsigned int);
  bool operator()(unsigned int, unsigned int) const;
};

#endif








