#include <vector>
#include "pcs.hpp"

using namespace std;

template<> 
void sol_set_add(domain_set_adaptor<bitset> &s, unsigned int l,
		 std::vector<variable_info> &v) {
  sol_set_add<bitset>(s,l,v);
}

template<> 
void sol_set_add(bitset &tmp, unsigned int label,
		 vector<variable_info> &varmap) {
  bitset result;
  bitset::iterator iend(tmp.end());
  for(bitset::iterator i(tmp.begin());
      i!=iend;++i) {
    unsigned int r(*i);
    // segfaults at next line when using vector_set.
    if(varmap[r].end <= (r+label)) {
      // otherwise ignore 
    } else {    
      result.insert(r+label);
    }
  }
  
  tmp.swap(result);
}

template<> 
void sol_set_add(invset<vector_set<unsigned int> > &tmp, unsigned int label,
		 vector<variable_info> &varmap) {
  typedef invset<vector_set<unsigned int> > invset_t;

  invset_t result(tmp.domain());
  invset_t::iterator iend(tmp.end());
  for(invset_t::iterator i(tmp.begin());
      i!=iend;++i) {
    unsigned int r(*i);
    // segfaults at next line when using vector_set.
    if(varmap[r].end <= (r+label)) {
      // otherwise ignore 
    } else {    
      result.insert(r+label);
    }
  }
  
  tmp.swap(result);
}


void condense_nodes(unsigned int n, unsigned int np, std::vector<variable_info> &varmap) {
  variable_info &to(varmap[n]);
  variable_info &src(varmap[np]);

  varmap[np].node = n;
  std::vector<unsigned int>::const_iterator k(src.variables.begin());
  std::vector<unsigned int>::const_iterator kend(src.variables.end());
  for(;k!=kend;++k) {
    varmap[*k].node = n;
  }

  to.variables.push_back(np);
  to.variables.insert(to.variables.end(),
		      src.variables.begin(),
		      src.variables.end());
  
  src.variables.clear();
  
  // finally, we move all complex constraints
  // out of np's set and into n's set
  
  vector_set<constraint> tmp;
  for(vector_set<constraint>::const_iterator i(src.complicates.begin());
      i!=src.complicates.end();++i) {
    if(i->rhs().dereferenced()) {
      constraint c(i->lhs(),expression(DEREFERENCE,n,i->rhs().offset()));
      tmp.insert(tmp.end(),c);
    } else {
      constraint c(expression(DEREFERENCE,n,i->lhs().offset()),i->rhs());
      tmp.insert(tmp.end(),c);
    }
  }

  set_union(to.complicates,tmp);

  src.complicates.clear();    
}




