#include <iostream>
#include <algorithm>
#include <sys/time.h>
#include <unistd.h>
#include "assert.h"
#include "bidirectional_adjacency_list.hpp"
#include "oto1_strategy.hpp"

using namespace std;
typedef bidirectional_adjacency_list<vector_set<unsigned int> > graph_t;

oto1_strategy::oto1_strategy(graph_t const &g) 
  : _n2i(size(g)), _visited(size(g),false), _marks(oto_comp(_n2i)) {
  // this should really by hidden in the dfs or topological sort
  // functions ...
  graph_change(g);
}

void oto1_strategy::push(unsigned int n) {
  _marks.insert(n);
}

void oto1_strategy::pop(void) {
  unsigned int s(_marks.size());
  _marks.erase(--_marks.end());
  if(_marks.size() != (s-1)) {
    cout << "WOAH PROBLEM IN POP!" << endl;
  }
}

bool oto1_strategy::is_marked(unsigned int n) const {
  return _marks.find(n) != _marks.end();
}

unsigned int oto1_strategy::top(void) const {
  return *_marks.rbegin();
}

bool oto1_strategy::empty(void) const {
  return _marks.empty();
}

void oto1_strategy::merge(unsigned int n, unsigned int np) {
  if(_marks.erase(np) > 0) { _marks.insert(n); }
}

void oto1_strategy::graph_change(graph_t const &g) {
  unsigned int index(0);
  for(unsigned int i=0;i!=size(g);++i) {
    if(!_visited[i]) {
      visit(i,index,g);
    }
  }
  fill(_visited.begin(),_visited.end(),false);
  set<unsigned int, oto_comp> tmp(_n2i);
  for(set<unsigned int, oto_comp>::iterator i(_marks.begin());
      i!=_marks.end();++i) {
    tmp.insert(*i);
  }

  tmp.swap(_marks);
}

void oto1_strategy::visit(unsigned int n, unsigned int &index, graph_t const &g) {
  _visited[n] = true;

  graph_t::const_siterator i,iend;
  for(boost::tie(i,iend) = successors(n,g);i!=iend;++i) {
    unsigned int w(*i);
    if(!_visited[w]) { visit(w,index,g); }
  }
  
  _n2i[n] = index++;
}









