#include <algorithm>
#include <sys/time.h>
#include <unistd.h>
#include "assert.h"
#include "oto_strategy.hpp"

using namespace std;
typedef bidirectional_adjacency_list<vector_set<unsigned int> > graph_t;

oto_strategy::oto_strategy(graph_t const &g) 
  : _priorities(size(g))
{
  unsigned int n(size(g));
  _timestamps.reserve(n);
  
  for(unsigned int i=0;i!=n;++i) {
    _timestamps.push_back(i);
  }    
}

void oto_strategy::push(unsigned int n) {
#if OTO_USE_MARKED == 1
  if(!_marked[n]) {
#endif
#if OTO_USE_STL_SET == 1
    _marks.insert(n);
#else
    vector<unsigned int>::iterator i
      (lower_bound(_marks.begin(),_marks.end(),n,*this));
    
    if(i == _marks.end() || *i != n) {
      _marks.insert(i,n);
    }
#endif
#if OTO_USE_MARKED == 1
    _marked[n]=true;
  }
#endif
}

void oto_strategy::pop(void) {
#if OTO_USE_STL_SET == 1
  unsigned int n(*_marks.rbegin());
  _timestamps[n]=_time++;
  _marks.erase(--_marks.end());
#else
  unsigned int n(_marks.back());
  _timestamps[n]=_time++;
  _marks.pop_back();
#endif
#if OTO_USE_MARKED == 1
  _marked[n]=false;
#endif
}

bool oto_strategy::is_marked(unsigned int n) const {
#if OTO_USE_STL_SET == 1
  return _marks.find(n) != _marks.end();
#else
  return binary_search(_marks.begin(),_marks.end(),n,*this);
#endif
}

unsigned int oto_strategy::top(void) const {
#if OTO_USE_STL_SET == 1
  return *_marks.rbegin();
#else
  return _marks.back();
#endif
}

bool oto_strategy::empty(void) const {
  return _marks.empty();
}

void oto_strategy::merge(unsigned int n, unsigned int np) {
#if OTO_USE_MARKED == 1
  if(_marked[np]) {
    _marked[np]=false;
#endif
#if OTO_USE_STL_SET == 1
    if(_marks.erase(np) > 0) { _marks.insert(n); }
#else
    vector<unsigned int>::iterator i
      (lower_bound(_marks.begin(),_marks.end(),np,*this));
    
    if(i != _marks.end() && *i == np) {
      _marks.erase(i);
      push(n);
    }    
#endif
#if OTO_USE_MARKED == 1
  }
#endif
}

bool oto_strategy::operator()(unsigned int a, unsigned int b) const {
  return _timestamps[a] > _timestamps[b];
}





