#ifndef SIMPLE_STORE_HPP
#define SIMPLE_STORE_HPP

#include <vector>
#include <algorithm>
#include "store.hpp"

template<class S>
class simple_store : public store<S> {
private:
  std::vector<S> _data;
  std::vector<bool> _cached;
  unsigned int _domain_size;
public:
  simple_store(unsigned int s, unsigned int d)
    : _data(s,d), _cached(s,false), _domain_size(d){
  }

  void clear(void) {
    std::fill(_cached.begin(),_cached.end(),false);
    for(typename std::vector<S>::iterator i(_data.begin());
	i!=_data.end();++i) {
      S empty(_domain_size);
      i->swap(empty); // clear contents using the "swap trick"
    }
  }

  void clear(unsigned int n) {
    _cached[n]=false;
    S empty(_domain_size);
    _data[n].swap(empty);
  }

  bool is_stored(unsigned int n) const { return _cached[n]; }

  bool equal_sets(unsigned int a, unsigned int b) const {
    return _data[a] == _data[b];
  }
  
  void update(unsigned int n, S const &data) {
    _cached[n]=true;
    _data[n] = data;
  }

  S const &operator[](unsigned int n) const { return _data[n]; }

  store<S> *clone(void) const { return new simple_store<S>(*this); }
};

#endif































