#include <algorithm>
#include <sys/time.h>
#include <unistd.h>
#include "assert.h"
#include "depth_first_search.hpp"
#include "sto2_strategy.hpp"

using namespace std;
typedef bidirectional_adjacency_list<set<unsigned int> > graph_t;

sto2_strategy::sto2_strategy(graph_t const &g) 
  : _n2i(size(g)) {
  // this should really by hidden in the dfs or topological sort
  // functions ...
  vector<bool> visited(size(g),false);
  sto2_visitor visitor(_n2i);
  for(unsigned int i=0;i!=size(g);++i) {
    if(!visited[i]) {
	depth_first_search<graph_t,vector<bool>,sto2_visitor&>(i,g,visited,visitor);
    }
  }
}

void sto2_strategy::push(unsigned int n) {
  if(!_current.empty() &&
     n < *_current.rbegin()) {
    std::vector<unsigned int>::iterator i
      (lower_bound(_next.begin(),_next.end(),n,*this));
    
    if(i == _next.end() || *i != n) {
      _next.insert(i,n);
    }
  } else {
    std::vector<unsigned int>::iterator i
      (lower_bound(_current.begin(),_current.end(),n,*this));
    
    if(i == _current.end() || *i != n) {
	_current.insert(i,n);
    }
  }
}

void sto2_strategy::pop(void) {
  _current.pop_back();
}
  
bool sto2_strategy::is_marked(unsigned int n) const {
  return binary_search(_current.begin(),_current.end(),n,*this) ||
    binary_search(_next.begin(),_next.end(),n,*this);
}
  
unsigned int sto2_strategy::top(void) const {
  if(_current.empty()) {
    _current = _next;
    _next.clear();
  }
  return _current.back();
}

bool sto2_strategy::empty(void) const {
  return _current.empty() && _next.empty();
}

void sto2_strategy::merge(unsigned int n, unsigned int np) {
  bool flag(false);
  std::vector<unsigned int>::iterator i
    (lower_bound(_current.begin(),_current.end(),np,*this));
  
  if(i != _current.end() && *i == np) {
    _current.erase(i);
    push(n);
    flag = true;
  }    
  
  i = lower_bound(_next.begin(),_next.end(),np,*this);
  
  if(i != _next.end() && *i == np) {
    _next.erase(i);
    if(!flag) { push(n); }
  }    
}

// used by the STL functions.
bool sto2_strategy::operator()(unsigned int a, unsigned int b) const {
  // notice I order the sets in reverse
  // order so that the next function is optimised
  return _n2i[a] > _n2i[b];
}



