#ifndef STO2_STRATEGY_HPP
#define STO2_STRATEGY_HPP

#include <set>
#include <vector>
#include <sys/time.h>
#include <unistd.h>
#include "assert.h"
#include "bidirectional_adjacency_list.hpp"

class sto2_strategy {
protected:
  class sto2_visitor {
  private:
    std::vector<unsigned int> &_n2i;
    unsigned int _counter;
  public:
    sto2_visitor(std::vector<unsigned int> &n2i) 
      : _n2i(n2i), _counter(n2i.size()) {
    }  

    inline void start_vertex(unsigned int n) {}
    inline void finished_vertex(unsigned int n) {
      assert(_counter < _n2i.size());
      _n2i[n] = --_counter;
    }
    inline void traversing_edge(unsigned int t, unsigned int h) {}
    inline void finished_edge(unsigned int t, unsigned int h) {}
    inline void skipping_edge(unsigned int t, unsigned int h) {}      
  };
  
  std::vector<unsigned int> _n2i; // node-to-topological index map
  mutable std::vector<unsigned int> _current;
  mutable std::vector<unsigned int> _next;
public:
  sto2_strategy(bidirectional_adjacency_list<std::set<unsigned int> > const &);
  void push(unsigned int);
  void pop(void);
  bool is_marked(unsigned int) const;
  unsigned int top(void) const;
  bool empty(void) const;
  void merge(unsigned int, unsigned int);
  bool operator()(unsigned int, unsigned int) const;
};

#endif





