#ifndef TOPOLOGICAL_SORT_HPP
#define TOPOLOGICAL_SORT_HPP

#include "depth_first_search.hpp"

template<class N2iMap>
class topological_sort_visitor {
 protected:
  unsigned int _counter;
  N2iMap &_n2i;
 public:
  topological_sort_visitor(N2iMap &n2i) 
    : _n2i(n2i), _counter(0) {
  }

  inline void start_vertex(unsigned int n) {}
  inline void finished_vertex(unsigned int n) {
    assert(_counter < _n2i.size());
    _n2i[_counter++]=n;
  }
  inline void traversing_edge(unsigned int t, unsigned int h) {}
  inline void finished_edge(unsigned int t, unsigned int h) {}
  inline void skipping_edge(unsigned int t, unsigned int h) {}
};

template<class Graph, class N2iMap>
void topological_sort(Graph const &g, N2iMap &map) {
  typedef topological_sort_visitor<N2iMap> Visitor;
  std::vector<bool> visited(size(g),false);
  Visitor visitor(map);
  for(unsigned int i=0;i!=size(g);++i) {
    if(!visited[i]) {
      depth_first_search<Graph,std::vector<bool>,Visitor&>(i,g,visited,visitor);
    }
  }
}

template<class Graph, class N2iMap, class Visited>
void topological_sort(Graph const &g, N2iMap &map, Visited &visited) {
  typedef topological_sort_visitor<N2iMap> Visitor;
  Visitor visitor(map);
  for(unsigned int i=0;i!=size(g);++i) {
    if(!visited[i]) {
      depth_first_search<Graph,std::vector<bool>,Visitor&>(i,g,visited,visitor);
    }
  }
}

#endif






