#ifndef SUIF_HELPERS_H
#define SUIF_HELPERS_H

#include <vector>
#include <string>
#include <suifkernel/suif_env.h>
#include <basicnodes/basic.h>
#include <suifnodes/suif.h>

typedef enum { NEAR, EXACT, UNKNOWN } line_number_t;

class line_number {
public:
  line_number_t type;
  unsigned int value;
public:
  line_number(line_number_t t, unsigned int v = 9999999) 
    : type(t), value(v) {
  }
};

bool is_pointer_type(Type *);
Type *dereference_type(Type *);
bool is_struct_type(Type *, StructType **);
bool is_pointer_to_struct(Type *, StructType **);
bool contains_pointer_type(Type *);
bool is_char_array_type(Type *);
unsigned int get_bit_offset(FieldSymbol *);
unsigned int get_field_number_from_bit_offset(unsigned int offset, StructType *);
unsigned int get_type_size_in_bytes(Type const *);
line_number get_line_number(AnnotableObject *);
unsigned int count_fields(StructType *);

ProcedureSymbol *is_static_call(Expression *);

/* binary opcodes */
extern int add_opcode;
extern int sub_opcode;
extern int mul_opcode;
extern int div_opcode;
extern int rem_opcode;
extern int band_opcode;
extern int bxor_opcode;
extern int bor_opcode;
extern int ls_opcode;
extern int rs_opcode;
extern int net_opcode;
extern int et_opcode;
extern int get_opcode;
extern int let_opcode;
extern int gt_opcode;
extern int lt_opcode;
extern int lor_opcode;
extern int land_opcode;

/* unary opcodes */
extern int con_opcode;
extern int lnot_opcode;
extern int bnot_opcode;
extern int neg_opcode;

#endif
